/* -*- mode: c++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */

/*
 Copyright (C) 2011 Klaus Spanderen

 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE.

 compile:
export LD_LIBRARY_PATH=/usr/lib/jvm/java-6-sun-1.6.0.24/jre/lib/i386/client
export CLASSPATH=/usr/local/lib/scala-library.jar:/usr/local/lib/scala-compiler.jar:.

g++ -O3 -I. -I/usr/local/include/boost-1_38 -I/home/spanderen/workspace/QuantLib -I/usr/lib/jvm/java-6-sun-1.6.0.24/include -I/usr/lib/jvm/java-6-sun-1.6.0.24/include/linux -o main main.cpp sandbox.cpp -L$LD_LIBRARY_PATH -ljvm -L/home/spanderen/workspace/QuantLib/ql/.libs -lQuantLib

scalac Payoff.scala InMemoryInterpreter.scala
javac ScalaBootLoader.java

*/


#include <sandbox.hpp>
#include <ql/settings.hpp>
#include <ql/time/daycounter.hpp>
#include <ql/quotes/simplequote.hpp>
#include <ql/math/randomnumbers/rngtraits.hpp>
#include <ql/time/daycounters/actualactual.hpp>
#include <ql/termstructures/yield/zerocurve.hpp>
#include <ql/math/statistics/generalstatistics.hpp>
#include <ql/methods/montecarlo/multipathgenerator.hpp>
#include <ql/pricingengines/vanilla/analytichestonengine.hpp>


using namespace QuantLib;

int main(int argc, char* argv[]) 
{
    const Date settlementDate(15, September, 2011);
    Settings::instance().evaluationDate() = settlementDate;

    const DayCounter dayCounter = ActualActual();
    const Date exerciseDate(15, September, 2012);
    const Time exerciseTime 
        = dayCounter.yearFraction(settlementDate, exerciseDate);

    const boost::shared_ptr<StrikedTypePayoff> payoff(
                                  new PlainVanillaPayoff(Option::Call, 100));
    const boost::shared_ptr<Exercise> exercise(
                                  new EuropeanExercise(exerciseDate));
    VanillaOption option(payoff, exercise);

    std::vector<Date> dates; 
    dates.push_back(settlementDate); dates.push_back(exerciseDate);
    std::vector<Rate> irates;
    irates.push_back(0.0); irates.push_back(0.02);
    Handle<YieldTermStructure> riskFreeTS(
            boost::shared_ptr<YieldTermStructure>(
                                    new ZeroCurve(dates, irates, dayCounter)));

    std::vector<Rate> qrates;
    qrates.push_back(0.0); qrates.push_back(0.03);
    Handle<YieldTermStructure> dividendTS(
            boost::shared_ptr<YieldTermStructure>(
                                    new ZeroCurve(dates, qrates, dayCounter)));
    
    const Real v0 = 0.1;
    const Real theta = 0.09;
    const Real kappa = 3.16;
    const Real sigma = 4.40;
    const Real rho = -0.8;

    Handle<Quote> s0(boost::shared_ptr<Quote>(new SimpleQuote(100)));

    boost::shared_ptr<HestonProcess> hestonProcess(
        new HestonProcess(riskFreeTS, dividendTS, s0, v0,
                          kappa, theta, sigma, rho));
    boost::shared_ptr<HestonModel> hestonModel(new HestonModel(hestonProcess));
    option.setPricingEngine(boost::shared_ptr<PricingEngine>(
                                    new AnalyticHestonEngine(hestonModel)));
    
    const Real expected = option.NPV();

    const TimeGrid grid(exerciseTime, 30u);
    typedef PseudoRandom::rsg_type rsg_type;
    typedef MultiPathGenerator<rsg_type>::sample_type sample_type;
    rsg_type rsg = PseudoRandom::make_sequence_generator(
        hestonProcess->size()*(grid.size()-1), BigNatural(421));
    MultiPathGenerator<rsg_type> generator(hestonProcess, grid, rsg, false);

    Sandbox::instance().compile("PayoffImpl.scala");
    
    GeneralStatistics stat;
    const DiscountFactor df = riskFreeTS->discount(exerciseTime);
    
    for (Size i=0; i < 20000; ++i) {
        const MultiPathGenerator<rsg_type>::sample_type::value_type& path 
            = generator.next().value;

        const Real npv = Sandbox::instance().payoff(
            std::vector<Real>(path[0].begin(), path[0].end())).back()*df;

        stat.add(npv);
    }

    printf("analytic price   : %5.3f\n",expected);
    printf("monte carlo price: %5.3f(%5.3f)\n",
           stat.mean(), stat.errorEstimate());

    return 0;
}

